/*******************************************************************************
* FILE NAME: OI_routines.c
*
* DESCRIPTION:
*    Update the OI display LEDs and the dashboard status information
*
*******************************************************************************/

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"

#include "sensor_routines.h"
#include "user_routines.h"
#include "motor_routines.h"
#include "print_routines.h"
//#include "tracker.h"
#include "clock_routines.h"


void Update_OI()
{
  char gyrovalue;
  int  igyrovalue;
  static long Last_Encoder=0;
  int itemp, itemp2;
  long ltemp;
  static unsigned long LastClock;
  long distance, velocity;
  static long LastDistance=0;

 /*---------- ROBOT FEEDBACK LEDs------------------------------------------------
  *------------------------------------------------------------------------------
  *   This section drives the "ROBOT FEEDBACK" lights on the Operator Interface.
  *   The lights are green for joystick forward and red for joystick reverse.
  *   Both red and green are on when the joystick is centered.  Use the
  *   trim tabs on the joystick to adjust the center.     
  */	
  
  if (user_display_mode == OFF) /* User Mode is Off */
    
  { /* Check trim position of Port 1 Joystick */

    if (ip1_y > -NullJoystick) 
      Pwm1_green  = ON;    /* Turn PWM1 green LED - OFF */
    else
      Pwm1_green  = OFF;    /* Turn PWM1 green LED - OFF */
    if (ip1_y <  NullJoystick) 
      Pwm1_red  = ON;      /* Turn PWM1 red LED   - ON */
    else
      Pwm1_red  = OFF;      /* Turn PWM1 red LED   - ON */


    if (ip4_x > -NullJoystick) 
      Pwm2_green  = ON;    /* Turn PWM1 green LED - OFF */
    else
      Pwm2_green  = OFF;    /* Turn PWM1 green LED - OFF */

    if (ip4_x <  NullJoystick) 
      Pwm2_red  = ON;      /* Turn PWM1 red LED   - ON */
    else
      Pwm2_red  = OFF;      /* Turn PWM1 red LED   - ON */

/* Check trim position of Port 1 Joystick */
    if (ip4_y > -NullJoystick) 
      Relay1_green  = ON;    /* Turn Relay1 green LED - OFF */
    else
      Relay1_green  = OFF;    /* Turn Relay1 green LED - OFF */

    if (ip4_y <  NullJoystick) 
      Relay1_red  = ON;      /* Turn Relay1 red LED   - ON */
    else
      Relay1_red  = OFF;      /* Turn Relay1 red LED   - ON */

/* Show which IR sensor detects the beacon */
//      switch (Tracker_Data[LEFT].Status)
//      {
//        case 0: //PrintString((char *) "None ");
//                Relay2_green = 0;
//                Relay2_red = 0;
//                break;
//        case 1: //PrintString((char *) "Right");
//                Relay2_green = 1;
//                Relay2_red = 0;
//                break;
//        case 2: //PrintString((char *) "Left ");
//                Relay2_green = 0;
//                Relay2_red = 1;
//                break;
//        case 3: //PrintString((char *) "Both ");
//                Relay2_green = 1;
//                Relay2_red = 1;
//                break;
//        default: //PrintString((char *) "FAULT");
//                Relay2_green = 0;
//                Relay2_red = 0;
//      }
  } /* (user_display_mode = 0) (User Mode is Off) */
  
  else  /* User Mode is On - displays data in OI 4-digit display*/
  {
//    User_Mode_byte = backup_voltage*10; /* so that decimal doesn't get truncated. */
    User_Mode_byte = PORTJ>>4;  // Autonomous selected
    if (rc_dig_in10)
      User_Mode_byte += 100; // Left
    else
      User_Mode_byte += 200; // Right
  }   
  Switch1_LED = rc_dig_in09; // Red/Blue switch - on = red
  Switch2_LED = GOAL_GRABBER; // Goal hook is down

  Switch3_LED = relay8_fwd; // Compressor on

  User_Byte1  = LeftAmps; // Left Amps
  User_Byte2  = RightAmps; // Right Amps

  /* Display the gyro heading byte3 if its negative, byte4 if its positive */
  igyrovalue = Gyro_Degrees();
  if (igyrovalue > 0)
  {
  User_Byte3  = 0; // Gyro degrees
  User_Byte4  = (char) igyrovalue;
  }
  else
  {
  User_Byte3  = (char) -igyrovalue; // Gyro degrees
  User_Byte4  = 0;
  }

  /* min/max the gyro has hit - to check for maxing out */
//  temp = -min_gyro/2 - 1;
//  User_Byte5  = (char) temp; // Turn rate minimum is -260
//  temp = max_gyro/2 - 1;
//  User_Byte6  = (char) temp; // Turn rate maximum is ~251


  if (LastClock < Clocksec)
  {
    LastClock = Clocksec;
// (2)encoders * 11.25"(one encoder rev) / 1(encoders) / 1000(3 decimal places) / 12(inches per foot)
  distance = (Right_Encoder_Count+ Left_Encoder_Count)/2 * 1125 / 25600; // inches
  ltemp = distance / 12; // Distance in feet
  if (ltemp > 255) ltemp = 255; 
  User_Byte5  = (char) ltemp; 

  /* velocity averaged over 1 sec. */
  velocity = distance - LastDistance; // inches per second
  LastDistance = distance;
  User_Byte6  = (char) velocity; // Velocity in ips

    PrintString((char *) ", Right_Encoder_Count=");
    PrintSignedDecimal(Right_Encoder_Count);
    PrintString((char *) ", Left_Encoder_Count=");
    PrintSignedDecimal(Left_Encoder_Count);
    PrintCR();
//    Right_Encoder_Count = 0;
//    Left_Encoder_Count = 0;
  }  
}
