/*******************************************************************************
* FILE NAME: auto_routines.c
*
*	Version 3.15	last updated 2/23/04::
*
* DESCRIPTION:
*  this is the autonomous mode place
*   this is where the autonomous mode is places  
*
* USAGE:
*  Just use it...
*
*******************************************************************************/

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"

#include "print_routines.h"
#include "motor_routines.h"
#include "auto_routines.h"
#include "user_routines.h"
#include "clock_routines.h"
#include "sensor_routines.h"
//#include "tracker.h"

#include "auto_scripts.h"  // All autonomous scripts reside here

#ifndef _FRC_BOARD
  unsigned char relay8_fwd,relay2_fwd,relay2_rev,relay4_fwd,relay4_rev;
#endif

/* Status is set by the individual auto functions */
static char Arm_Status=COMPLETE, Drive_Status=COMPLETE, Joint_Status=COMPLETE;
static int lastDriveCmd=ENDCMD, lastArmCmd=ENDCMD, lastJointCmd=ENDCMD;
static unsigned char DriveCmd=0, ArmCmd=0, JointCmd=0; // Current cmd # in script


/*******************************************************************************
* FUNCTION NAME: User_Autonomous_Code
* PURPOSE:       this is called constantly and checks to see if it is time to
*				  execute a function. (This is Autonomous mode)
* CALLED FROM:   main.c
* ARGUMENTS:     void
* RETURNS:       moves the robot based on how it is set.
*******************************************************************************/
void User_Autonomous_Code(void)
{
  static unsigned int swtc;  //set equal to the switch input
  static char FirstTime=TRUE;

//  #define REDALLIANCE   rc_dig_in09  //OFF=Red=TRUE, ON=Blue=FALSE
  #define LEFTFIELD     rc_dig_in10  //OFF=Left=TRUE, ON=Right=FALSE


  /* 09 = RED/BLUE Alliance */
  /* 10 = LEFT/RIGHT of field */
  /* When red, red mobile goal is on your right, & beacon type-1 is by the red mobile goal */
//  if ( REDALLIANCE &&  LEFTFIELD)  Beacon_type = BEACON_TYPE_0;
//  if ( REDALLIANCE && !LEFTFIELD)  Beacon_type = BEACON_TYPE_1;
//  if (!REDALLIANCE &&  LEFTFIELD)  Beacon_type = BEACON_TYPE_1;
//  if (!REDALLIANCE && !LEFTFIELD)  Beacon_type = BEACON_TYPE_0;

  /** Autonomous is selected by the left/right field switch and the auto mode selector **/
  /* Only check this switch once at the start */
  if (FirstTime)
  {
    FirstTime = FALSE;
    swtc = PORTJ>>4;  // Auto select switch 1-16
//    GOAL_GRABBER = 0; // Make sure goal grabber is up
//    relay3_rev = 0;
    AutoGoal = FALSE; // Leave goal in position until first user trigger

#ifndef _FRC_BOARD
    swtc =15; //DEBUG ONLY
#endif
  } /* End FirstTime */

//  LEFTFIELD is used within certain functions to reverse certain script cmd parameters
//  mirroring operations on the opposite side of the field
//  Commands affected include: DSPIN, DISTTURN, GTURN, GTURNGOAL
    // Some cmd scripts objectives don't mirror, like pushing the other alliance's
    // goal under the ball drop

    /* Copy selected script into workspace */
    switch(swtc)
	  {
		  case 0: 
	    default:
        Switch_Auto(AUTO_0);
			  break;
  		case 1: 
        Switch_Auto(AUTO_1);
		  	break;
  		case 2: 
        Switch_Auto(AUTO_2);
		  	break;
  		case 3: 
        Switch_Auto(AUTO_3);
		  	break;
  		case 4: 
        Switch_Auto(AUTO_4);
  	  	break;
  		case 5: 
        Switch_Auto(AUTO_5);
		  	break;
  		case 6: 
        Switch_Auto(AUTO_6);
		  	break;
  		case 7: 
        Switch_Auto(AUTO_7);
		  	break;
  		case 8: 
        Switch_Auto(AUTO_8);
		  	break;
  		case 9: 
        Switch_Auto(AUTO_9);
		  	break;
  		case 10: 
        Switch_Auto(AUTO_10);
		  	break;
  		case 11: 
        Switch_Auto(AUTO_11);
		  	break;
  		case 12: 
        Switch_Auto(AUTO_12);
		  	break;
  		case 13: 
        Switch_Auto(AUTO_13);
		  	break;
  		case 14: 
        Switch_Auto(AUTO_14);
		  	break;
  		case 15: 
        Switch_Auto(AUTO_15);
		  	break;
	   }
  /* 4 motor drive */
  WHL_L2 = WHL_L1;	//set the other motor to same
  WHL_R2 = WHL_R1;  //same note as above...

  /** Perform housekeeping functions **/
  COMPRESSOR = !rc_dig_in14;  /* Power pump only if pressure switch is off. */
  Update_OI(); // Update the OI LED & dashboard displays
}


/*******************************************************************************
* FUNCTION NAME: Switch_Auto
* PURPOSE:       Process an autonomous command script
* ARGUMENTS:     Command script
* RETURNS:       moves the robot based on the command script.
*******************************************************************************/
void Switch_Auto(rom const int *CommandScript)
{
  static unsigned char cmdNum=0,lastcmdNum=99;
  int speed, distance, time, direction, heading, distance_right, distance_left;
  int setting, nextCmd;
  int i, j, k;

  /* Process existing commands - */
  /* Original command arguments are preserved within the functions themselves */

  /*******************************/
  /************ Drive ************/
  switch(lastDriveCmd)
  {
      default:
      case ENDCMD:  // Do nothing - no drive cmds yet
          break;    
      case DFWD:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Forward_Time(time, speed);
          break;
      case DBCK:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Backward_Time(time, speed);
          break;
        case DSPIN:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          direction = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIRECTION];
          Drive_Spin_In_Place_Time(time, speed, direction);
          break;
      case DISTFWD:
          distance = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIST];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Forward_Dist(distance, speed);
          break;
      case DISTBCK:
          distance = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIST];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Backward_Dist(distance, speed);
          break;
      case GOALGRAB:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          Goal_Grab(time);
          break;
      case GOALREL:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          Goal_Release(time);
          break;
      case GTURN:
          heading = CommandScript[DriveCmd*AUTO_WIDTH + CMDGHEADING];
          Gyro_Turn(heading);
          break;
      case BALANCE:
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Balance(speed);
          break;
  }
  /*******************************/
  /************** Arm   **********/
  switch(lastArmCmd)
  {
     default:
     case ENDCMD:  // Do nothing - no arm cmds yet
          break;    
      case ARMOPN:
          Arm_Open();
          break;
      case ARMCLS:
          Arm_Close();
          break;
      case ARMFWD:
          time = CommandScript[ArmCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[ArmCmd*AUTO_WIDTH + CMDSPEED];
          Arm_Forward_Time(time, speed);
          break;
      case ARMBCK:
          time = CommandScript[ArmCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[ArmCmd*AUTO_WIDTH + CMDSPEED];
          Arm_Backward_Time(time, speed);
          break;
      case ARMSET:
          setting = CommandScript[ArmCmd*AUTO_WIDTH + CMDSETTING];
          Arm_Setting(setting);
          break;
  }

  /*******************************/
  /************* Joint ***********/
  switch(lastJointCmd)
  {
      default:
      case ENDCMD:  // Do nothing - no joint cmds yet
          break;    
      case DELAY:
          time = CommandScript[JointCmd*AUTO_WIDTH + CMDTIME];
          Delay(time);
          break;
      case SYNC:
          time = CommandScript[JointCmd*AUTO_WIDTH + CMDTIME];
          Sync_Drive_Arm(time);
          break;
  }

/******************************/

  /* Look at next command on each queue */
  nextCmd = CommandScript[cmdNum*AUTO_WIDTH + CMD];
//  if (cmdNum != lastcmdNum)
//  {
//    lastcmdNum = cmdNum;
//    PrintString((char *) " --cmdNum=");
//    PrintDecimal(cmdNum);
//    PrintString((char *) ", nextCmd=");
//    PrintDecimal(nextCmd);
//    PrintString((char *) ", Drive_Status=");
//    PrintDecimal(Drive_Status);
//    PrintString((char *) ", lastDriveCmd=");
//    PrintDecimal(lastDriveCmd);
//    PrintString((char *) "-- ");
//  }

  if (nextCmd != ENDCMD)
  {
   if (nextCmd <= GTURN) // It's a Drive command
   {
     if (Drive_Status == COMPLETE)
    {    /* Start next Drive cmd */
      DriveCmd = cmdNum;
      cmdNum++;  // Get ready for next command
      switch(nextCmd)
      { 
        default:
        case ENDCMD:  // Do nothing - no drive cmds yet
          break;    
        case DFWD:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Forward_Time(time, speed);
          break;
        case DBCK:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Backward_Time(time, speed);
          break;
        case DSPIN:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          direction = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIRECTION];
          Drive_Spin_In_Place_Time(time, speed, direction);
          break;
        case DISTFWD:
          distance = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIST];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Forward_Dist(distance, speed);
          break;
        case DISTBCK:
          distance = CommandScript[DriveCmd*AUTO_WIDTH + CMDDIST];
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Drive_Backward_Dist(distance, speed);
          break;
        case GOALGRAB:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          Goal_Grab(time);
          break;
        case GOALREL:
          time = CommandScript[DriveCmd*AUTO_WIDTH + CMDTIME];
          Goal_Release(time);
          break;
        case GTURN:
          heading = CommandScript[DriveCmd*AUTO_WIDTH + CMDGHEADING];
          Gyro_Turn(heading);
          break;
      case BALANCE:
          speed = CommandScript[DriveCmd*AUTO_WIDTH + CMDSPEED];
          Balance(speed);
          break;
      }
     }
    }
    else if (nextCmd >= ARMOPN)
    {
     if (Arm_Status == COMPLETE)
     {    /* Start next Arm cmd */
      ArmCmd = cmdNum;
      cmdNum++;  // Get ready for next command
      switch(nextCmd)
      {
        default:
        case ENDCMD:  // Do nothing - no arm cmds yet
          break;    
        case ARMOPN:
          Arm_Open();
          break;
        case ARMCLS:
          Arm_Close();
          break;
        case ARMFWD:
          time = CommandScript[ArmCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[ArmCmd*AUTO_WIDTH + CMDSPEED];
          Arm_Forward_Time(time, speed);
          break;
        case ARMBCK:
          time = CommandScript[ArmCmd*AUTO_WIDTH + CMDTIME];
          speed = CommandScript[ArmCmd*AUTO_WIDTH + CMDSPEED];
          Arm_Backward_Time(time, speed);
          break;
        case ARMSET:
          setting = CommandScript[ArmCmd*AUTO_WIDTH + CMDSETTING];
          Arm_Setting(setting);
          break;
      }
     }
    }
    else if (Joint_Status == COMPLETE)
    {    /* Start next Joint cmd */
      JointCmd = cmdNum;
      cmdNum++;  // Get ready for next command
      switch(nextCmd)
      {
        default:
        case ENDCMD:  // Do nothing - no joint cmds yet
          break;    
        case DELAY:
          time = CommandScript[JointCmd*AUTO_WIDTH + CMDTIME];
          Delay(time);
          break;
        case SYNC:
          time = CommandScript[JointCmd*AUTO_WIDTH + CMDTIME];
          Sync_Drive_Arm(time);
          break;
      }
    }
  } /* End ENDCMD check */
}



/*******************************************************************************
* FUNCTION NAME: abs
* PURPOSE:       Return absolute value
*******************************************************************************/
unsigned int abs(signed int inb) 
{
  if(inb >= 0)
      return inb;
  else
      return -inb;
}


/*******************************************************************************
* FUNCTION NAME: Gyro_Turn
* PURPOSE:       Turn the robot to the specified heading.
* CALLED FROM:   file:user_routines_fast.c, here
* ARGUMENTS:     input: speed 0-127
* RETURNS:       void
*******************************************************************************/
#define PWRFACTOR 15   // Reduce power as we approach target heading (40 works)
                       // was 28

void Gyro_Turn(int heading)
{
  int gyrodegrees, temp;
  static int our_heading;
  static char overshoot=0; // Detect over corrections

  if (lastDriveCmd != GTURN)
  {
    lastDriveCmd = GTURN;
    our_heading = heading;
    PrintString((char *) " **Gyro_Turn** ");
    PrintCR();
    /** Assume heading is based on left field action **/
    /** Reverse if robot is on the right side of the field **/
    if (!LEFTFIELD) // TRUE = left
    {
      PrintString((char *) "***** RIGHT field side ******");
      our_heading = -our_heading;
    }

    /** need to wrap heading properly **/
    if (our_heading > 180)
      our_heading = 179;
    else if (our_heading < -180)
      our_heading = -179;
    else if (our_heading == 180) // Mathematically we just never get exactly 180 degrees
      our_heading = 179;  // Use the next closest value to 180
  }

  gyrodegrees = Gyro_Degrees();
  /** Convert gyrodegrees to be as close as possible to heading **/
  /*  This determines the direction the robot will spin */
  /*  This avoids issues with wrapping over the 180 degree mark (-179 + 1 = 180)*/
  if (gyrodegrees > (our_heading+180)) gyrodegrees -= 360;
  if (gyrodegrees < (our_heading-180)) gyrodegrees += 360;

  if (our_heading == gyrodegrees)
  {
    WHL_L1 = NEUTRAL;
    WHL_R1 = NEUTRAL;
    overshoot = 0;
    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    Drive_Status = COMPLETE;
PrintString((char *) " gyro is done ");
  }
  else
  {
    Drive_Status = INCOMPLETE;
    if (our_heading > gyrodegrees) /* Spin Counter Clockwise - when gyro is upright */
    {
//      WHL_L1 = -FULLSPEED * (heading-gyrodegrees) / PWRFACTOR;
      WHL_L1 = FULLSPEED * (gyrodegrees-our_heading) / PWRFACTOR; // negative value
      if (WHL_L1 < -FULLSPEED) WHL_L1 = -FULLSPEED;
      if (WHL_L1 > -MINSPEED) WHL_L1 = -MINSPEED;

      if (overshoot == 2) // Detect overshoot & stop
      {
        WHL_L1 = NEUTRAL;
        WHL_R1 = NEUTRAL;
        overshoot = 0;
        lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
        Drive_Status = COMPLETE;
      }
      overshoot=1;
    }
    else /* Spin the opposite direction - Clockwise */
    {
      WHL_L1 = FULLSPEED * (gyrodegrees-heading) / PWRFACTOR; // Positive value
      if (WHL_L1 > FULLSPEED) WHL_L1 = FULLSPEED;
      if (WHL_L1 < MINSPEED) WHL_L1 = MINSPEED;

      if (overshoot == 1) // Detect overshoot & stop
      {
        WHL_L1 = NEUTRAL;
        WHL_R1 = NEUTRAL;
        overshoot = 0;
        lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
        Drive_Status = COMPLETE;
      }
      overshoot=2;
    }
  }
  PrintString((char *) " gyro = ");
  PrintSignedDecimal(gyrodegrees);
  PrintString((char *) " our_heading = ");
  PrintSignedDecimal(our_heading);
  PrintCR();
}


/*******************************************************************************
* FUNCTION NAME: Drive_Forward_Time
* PURPOSE:       Drive straight for a given time at a given speed.
*                Pure time 1 sec = 87", 2 sec = 199.5" = top speed 9.38 fps
*                Each function determines completion or failure.
* SENSORS:       Gyro - for keeping straight
*******************************************************************************/
void Drive_Forward_Time(int time, int speed)
{
  int desired_heading;
  char init;
  static unsigned long duration; //in tenths of seconds

 
  if (lastDriveCmd != DFWD)
  {
    lastDriveCmd = DFWD;
    duration = Clockcs + time; // Current time + duration = cmd completed
    /* Keep straight using the gyro */
    init = TRUE;
    desired_heading = Gyro_Degrees();
    Gyro_PID(desired_heading, init);  // Establish desired heading
    PrintString((char *) " **Drive_Forward_Time** ");
    PrintDecimal(Clockcs);
    PrintCR();
  }

  if (duration > Clockcs)
  { // Keep going
    Drive_Status = INCOMPLETE;
    WHL_L1 = -speed;
    WHL_R1 = -speed;
    WHL_L1 = WHL_L1 - (WHL_L1/127 * FORWARDLEFTBALANCE);  // dead reckoning balance
    WHL_R1 = WHL_R1 - (WHL_R1/127 * FORWARDRIGHTBALANCE);
    init = FALSE; // NOT the first time
//    Gyro_PID(desired_heading, init); // Adjusts power to motors to keep a straight heading
    PrintString((char *) " WHL_L1(-127)=");
    PrintSignedDecimal(WHL_L1);
    PrintString((char *) " WHL_R1=");
    PrintSignedDecimal(WHL_R1);
  }
  else
  { // All done (move on to next drive command)
    Drive_Status = COMPLETE;
    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    WHL_L1 = NEUTRAL;
    WHL_R1 = NEUTRAL;
  }
}

/*******************************************************************************
* FUNCTION NAME: Drive_Backward_Time
* PURPOSE:       Function-based autonomous mode.
* CALLED FROM:   file:user_routines_fast.c, here
* ARGUMENTS:     input: speed 0-127
*                output: 
* RETURNS:       void
*******************************************************************************/
void Drive_Backward_Time(int time, int speed)
{
  int desired_heading;
  char init;
  static unsigned long duration; //in tenths of seconds


  if (lastDriveCmd != DBCK)
  {
    lastDriveCmd = DBCK;
    duration = Clockcs + time; // Current time + duration = cmd completed
    /* Keep straight using the gyro */
    init = TRUE;
    desired_heading = Gyro_Degrees();
    Gyro_PID(desired_heading, init);  // Establish desired heading
    PrintString((char *) " **Drive_Backward_Time** ");
    PrintDecimal(Clockcs);
    PrintCR();
  }

  if (duration > Clockcs)
  { // Keep going
    Drive_Status = INCOMPLETE;
    WHL_L1 = speed;
    WHL_R1 = speed;
    WHL_L1 = WHL_L1 - (WHL_L1/127 * BACKWARDLEFTBALANCE);  // dead reckoning balance
    WHL_R1 = WHL_R1 - (WHL_R1/127 * BACKWARDRIGHTBALANCE);
    init = FALSE; // NOT the first time
//    Gyro_PID(desired_heading, init); // Adjusts power to motors to keep a straight heading
    PrintString((char *) " WHL_L1(127)=");
    PrintSignedDecimal(WHL_L1);
    PrintString((char *) " WHL_R1=");
    PrintSignedDecimal(WHL_R1);
  }
  else
  { // All done (move on to next drive command)
    Drive_Status = COMPLETE;
    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    WHL_L1 = NEUTRAL;
    WHL_R1 = NEUTRAL;
  }
}



/*******************************************************************************
* FUNCTION NAME: Drive_Spin_In_Place_Time
* PURPOSE:       Function-based autonomous mode.
* CALLED FROM:   file:user_routines_fast.c, here
*******************************************************************************/
void Drive_Spin_In_Place_Time(int time, int speed, int direction)
{
  static unsigned long duration; //in tenths of seconds

 
  if (lastDriveCmd != DSPIN)
  {
    lastDriveCmd = DSPIN;
    Drive_Status = INCOMPLETE;
    duration = Clockcs + time; // Current time + duration = cmd completed
    /** Assume direction is based on left field action **/
    /** Reverse if robot is on the right side of the field **/
    if (!LEFTFIELD) // TRUE = left
    {
      if (direction == CW) // Reverse direction
         direction = CCW;
      else
         direction = CW;
    }
    PrintString((char *) " **Drive_Spin_In_Place_Time** ");
    PrintCR();
  }

  if (duration > Clockcs)
  { // Keep going
    Drive_Status = INCOMPLETE;

    if (direction == CW) /* Spin Clockwise */
    {
      WHL_L1 = -speed;
      WHL_R1 = speed;

    }else /* Spin the opposite direction - Counter Clockwise */
    {
      WHL_L1 = speed;
      WHL_R1 = -speed;
    }
  }
  else
  { // All done (move on to next drive command)
    Drive_Status = COMPLETE;
    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    WHL_L1 = NEUTRAL;
    WHL_R1 = NEUTRAL;
  }
}


/*******************************************************************************
* FUNCTION NAME: Drive_Forward_Dist
* PURPOSE:       Simple drive straight function for function-based autonomous mode.
*                Top speed @ full weight on carpet is 9.38 fps
*******************************************************************************/
void Drive_Forward_Dist(int dist, int speed)
{
  static unsigned long desired_distance; // in inches
  long encoder_dist;
  char init;
  static int desired_heading;


  if (lastDriveCmd != DISTFWD)
  {
    lastDriveCmd = DISTFWD;
    PrintString((char *) " **Drive_Forward_Dist** ");
    PrintCR();

    /* Disable interrupts while we zero the shaft encoders */
    INTCONbits.GIEL = 0;
    Right_Encoder_Count = 0;
    Left_Encoder_Count = 0;
    INTCONbits.GIEL = 1;
    
    desired_distance = (unsigned long) dist;
    desired_distance = desired_distance * (unsigned long)25600 / (unsigned long)1125; // Convert from inches to encoder units
    
    init = TRUE;
    desired_heading = Gyro_Degrees();
    Gyro_PID(desired_heading, init);  // Initialize desired heading
  }

  encoder_dist = (Right_Encoder_Count + Left_Encoder_Count) / 2;
  if (encoder_dist < 0) encoder_dist = -encoder_dist;  /* Use absolute values */

  PrintString((char *) " dist=");
  PrintSignedDecimal(dist);
  PrintString((char *) " encoder_dist=");
  PrintSignedDecimal(encoder_dist);
  PrintString((char *) " desired_distance=");
  PrintSignedDecimal(desired_distance);
  PrintCR();

  if (desired_distance > encoder_dist)
  {
    Drive_Status = INCOMPLETE;

//    if ((desired_distance - encoder_dist) < SLOWDOWN) 
//    { // When close to target encoder count start slowing down
//      /* As we get closer slow down so we don't overshoot the target by quite so much (2.5' @ full speed)*/
//      /* Apply proportional slow down */
//      WHL_L1 = speed * (desired_distance - encoder_dist) / SLOWDOWN; // Positive value
//      if (WHL_L1 > FULLSPEED) WHL_L1 = FULLSPEED;
//      if (WHL_L1 < MINSPEED) WHL_L1 = MINSPEED;
//      WHL_L1 = -WHL_L1;
//      WHL_R1 = WHL_L1;
//    }
//    else
    { // Full speed
      WHL_L1 = -speed;
      WHL_R1 = WHL_L1;
    }
    /* Balance the motors */
    WHL_L1 = WHL_L1 - (WHL_L1/127 * FORWARDLEFTBALANCE);  // dead reckoning balance
    WHL_R1 = WHL_R1 - (WHL_R1/127 * FORWARDRIGHTBALANCE);
    init = FALSE;
    Gyro_PID(desired_heading, init); // Fine tune desired heading

    PrintString((char *) "WHL_L1=");
    PrintSignedDecimal(WHL_L1);
    PrintString((char *) "WHL_R1=");
    PrintSignedDecimal(WHL_R1);
  }
  else
  {
    WHL_L1 = NEUTRAL;
    WHL_R1 = NEUTRAL;
    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    Drive_Status = COMPLETE;
  }
}

/*******************************************************************************
* FUNCTION NAME: Drive_Backward_Dist
* PURPOSE:       Simple drive straight function for function-based autonomous mode.
*                Top speed @ full weight on carpet is 9.38 fps
*******************************************************************************/
void Drive_Backward_Dist(int dist, int speed)
{
  static unsigned long desired_distance; // in inches
  long encoder_dist;
  char init;
  static int desired_heading;


  if (lastDriveCmd != DISTBCK)
  {
    lastDriveCmd = DISTBCK;
    PrintString((char *) " **Drive_Backward_Dist** ");
    PrintCR();

    /* Disable interrupts while we zero the shaft encoders */
    INTCONbits.GIEL = 0;
    Right_Encoder_Count = 0;
    Left_Encoder_Count = 0;
    INTCONbits.GIEL = 1;
    
    desired_distance = (unsigned long) dist;
    desired_distance = desired_distance * (unsigned long)25600 / (unsigned long)1125; // Convert from inches to encoder units
    
    init = TRUE;
    desired_heading = Gyro_Degrees();
    Gyro_PID(desired_heading, init);  // Initialize desired heading
  }

  encoder_dist = (Right_Encoder_Count + Left_Encoder_Count) / 2;
  if (encoder_dist < 0) encoder_dist = -encoder_dist;  /* Use absolute values */

  PrintString((char *) " dist=");
  PrintSignedDecimal(dist);
  PrintString((char *) " encoder_dist=");
  PrintSignedDecimal(encoder_dist);
  PrintString((char *) " desired_distance=");
  PrintSignedDecimal(desired_distance);
  PrintCR();

  if (desired_distance > encoder_dist)
  {
    Drive_Status = INCOMPLETE;

//    if ((desired_distance - encoder_dist) < SLOWDOWN) 
//    { // When close to target encoder count start slowing down
//      /* As we get closer slow down so we don't overshoot the target by quite so much (2.5' @ full speed)*/
//      /* Apply proportional slow down */
//      WHL_L1 = speed * (desired_distance - encoder_dist) / SLOWDOWN; // Positive value
//      if (WHL_L1 > FULLSPEED) WHL_L1 = FULLSPEED;
//      if (WHL_L1 < MINSPEED) WHL_L1 = MINSPEED;
//      WHL_L1 = -WHL_L1;
//      WHL_R1 = WHL_L1;
//    }
//    else
    { // Full speed
      WHL_L1 = -speed;
      WHL_R1 = WHL_L1;
    }
    /* Balance the motors */
    WHL_L1 = WHL_L1 - (WHL_L1/127 * FORWARDLEFTBALANCE);  // dead reckoning balance
    WHL_R1 = WHL_R1 - (WHL_R1/127 * FORWARDRIGHTBALANCE);
    init = FALSE;
    Gyro_PID(desired_heading, init); // Fine tune desired heading

    PrintString((char *) "WHL_L1=");
    PrintSignedDecimal(WHL_L1);
    PrintString((char *) "WHL_R1=");
    PrintSignedDecimal(WHL_R1);
  }
  else
  {
    WHL_L1 = NEUTRAL;
    WHL_R1 = NEUTRAL;
    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    Drive_Status = COMPLETE;
  }
}


/*******************************************************************************
* FUNCTION NAME: Arm_Open
* PURPOSE:       Simple open arms function for function-based autonomous mode.
*******************************************************************************/
void Arm_Open(void)
{
if (lastDriveCmd != ARMOPN)
{
  PrintString((char *) " **Open_Arms** ");
  PrintCR();
} 
  BALL2X_GRABBER = 1;
  relay2_rev = 0;  // Not necessary only here as a future reminder
  lastArmCmd = ENDCMD;  // In case there are two of the same type cmd in a row
  Arm_Status = COMPLETE;

}

/*******************************************************************************
* FUNCTION NAME: Arm_Close
* PURPOSE:       Simple close arms function for function-based autonomous mode.
*******************************************************************************/
void Arm_Close(void)
{
  if (lastDriveCmd != ARMCLS)
  {
    PrintString((char *) " **Close_Arms** ");
    PrintCR();
  } 
  BALL2X_GRABBER = 0;
  relay2_rev = 0; // Not necessary as long as it's been initialized to 0 somewhere
  lastArmCmd = ENDCMD;  // In case there are two of the same type cmd in a row
  Arm_Status = COMPLETE;
}

/*******************************************************************************
* FUNCTION NAME: Arm_Forward
* PURPOSE:       Simple arms forward function for function-based autonomous mode.
*
* Notes:         arms down in starting position
*                ?=10pt ball height
*                ?=mobile goal 2x ball height
*                ?=stationary goal 2x ball height
*                ?=straight up
*******************************************************************************/
void Arm_Forward_Time(int time, int speed)
{
  static unsigned long duration;

 
  if (lastArmCmd != ARMFWD)
  {
    lastArmCmd = ARMFWD;
    duration = Clockcs + time; // Current time + duration = cmd completed
    PrintString((char *) " **Arms_Forward** ");
    PrintCR();
  }

  if (duration > Clockcs)
  { // Keep going
	  ARM_PWM = speed;
    Arm_Status = INCOMPLETE;
  }
  else
  { // All done (move on to next drive command)
	  ARM_PWM = NEUTRAL;
    Arm_Status = COMPLETE;
    lastArmCmd = ENDCMD;  // In case there are two of the same type cmd in a row
  }
}


/*******************************************************************************
* FUNCTION NAME: Arm_Backward
* PURPOSE:       Simple arms back function for function-based autonomous mode.
*
* Notes:         arms will be coming from any position
*******************************************************************************/
void Arm_Backward_Time(int time, int speed)
{
  static unsigned long duration;

 
  if (lastArmCmd != ARMBCK)
  {
    lastArmCmd = ARMBCK;
    duration = Clockcs + time; // Current time + duration = cmd completed
    PrintString((char *) " **Arms_Backward** ");
    PrintCR();
  }

  if (duration > Clockcs)
  { // Keep going
	  ARM_PWM = -speed;
    Arm_Status = INCOMPLETE;
  }
  else
  { // All done (move on to next drive command)
	  ARM_PWM = NEUTRAL;
    Arm_Status = COMPLETE;
    lastArmCmd = ENDCMD;  // In case there are two of the same type cmd in a row
  }
}


/*******************************************************************************
* FUNCTION NAME: Arm_Setting
* PURPOSE:       Position arms to the selected banner encoder setting
* ARGUMENTS:     setting - count of banner encoder stripes from the start position.
* RETURNS:       Status - COMPLETE if function has finished requested task
*                         INCOMPLETE if end conditions have not yet been satisfied
*
* Notes: settings - 0=arms down in starting position
*                   ?=10pt ball height
*                   ?=mobile goal 2x ball height
*                   ?=stationary goal 2x ball height
*                   ?=straight up
*******************************************************************************/
void Arm_Setting(int setting)
{
  if (lastArmCmd != ARMSET)
  {
    lastArmCmd = ARMSET;
    Arm_Status = INCOMPLETE;
    PrintString((char *) " **Arm_Setting** ");
    PrintCR();
  } 

  /* Include a proportional slow down */
  if (setting > ArmSetting)
  { // Forward
    if ((setting-2) <= ArmSetting)  // Slow down if within 2 settings of the target
	    ARM_PWM = HALFSPEED;
    else
	    ARM_PWM = FULLSPEED;
  }
  else if (setting < ArmSetting)
  { // Backward
    if ((setting+2) >= ArmSetting)  // Slow down if within 2 settings of the target
	    ARM_PWM = -HALFSPEED;
    else
	    ARM_PWM = -FULLSPEED;
  }
  else
  {
	  ARM_PWM = NEUTRAL;
    lastArmCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    Arm_Status = COMPLETE;
  }
}


/*******************************************************************************
* FUNCTION NAME: Goal_Grab
* PURPOSE:       Activate goal latch
*******************************************************************************/
void Goal_Grab(int time)
{
  static unsigned long duration; //in tenths of seconds

  if (lastDriveCmd != GOALGRAB)
  {
    lastDriveCmd = GOALGRAB;
    duration = Clockcs + time; //  Delay for one second to allow for drive coasting
    Drive_Status = INCOMPLETE;
    PrintString((char *) " **Goal_Grab** ");
    PrintCR();
  }

  if (duration < Clockcs)
  {
    GOAL_GRABBER = 1;
    relay4_rev = 0;  // Not required but should be set as a matter of principle
    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    Drive_Status = COMPLETE;
  }
}


/*******************************************************************************
* FUNCTION NAME: Goal_Search
* PURPOSE:       Go Forward slowly until mobil goal detection switch, then
*                Activate goal latch. If unsuccessful then try again.
*                 If still unsuccessful then return to position at start of maneauver
*******************************************************************************/
void Goal_Search(int time)
{
  static unsigned long duration; //in tenths of seconds

  if (lastDriveCmd != GOALSRC)
  {
    lastDriveCmd = GOALSRC;
    duration = Clockcs + time; //  Delay for one second to allow for drive coasting
    Drive_Status = INCOMPLETE;
    PrintString((char *) " **Goal_Grab** ");
    PrintCR();
  }

  if (duration < Clockcs)  // Wait to start
  {
//    if (GOAL_DETECT)
    {
      GOAL_GRABBER = 1;
      relay4_rev = 0;  // Not required but should be set as a matter of principle
    }

    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    Drive_Status = COMPLETE;
  }
}


/*******************************************************************************
* FUNCTION NAME: Goal_Release
* PURPOSE:       Activate goal latch
*******************************************************************************/
void Goal_Release(int time)
{
  static unsigned long duration; //in tenths of seconds

  if (lastDriveCmd != GOALGRAB)
  {
    lastDriveCmd = GOALGRAB;
    duration = Clockcs + time; //  Delay for one second to allow for drive coasting
    Drive_Status = INCOMPLETE;
    PrintString((char *) " **Goal_Release** ");
    PrintCR();
  }

  if (duration < Clockcs)
  {
    GOAL_GRABBER = 0;
    relay4_rev = 0;
    lastDriveCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    Drive_Status = COMPLETE;
  }
}


/*******************************************************************************
* FUNCTION NAME: Delay
* PURPOSE:       Delay for a period of time in tenths of seconds
*                There is an implied SYNC here. This will pause all operations.
*******************************************************************************/
void Delay(int time)
{
  static unsigned long duration;
  static char startFlag=TRUE;

  if (lastJointCmd != DELAY)
  {
    lastJointCmd = DELAY;
    Joint_Status = INCOMPLETE;
    PrintString((char *) " **Delay** ");
    PrintCR();
  }

  /* Don't start until both drive and arm command queues are free */
  if (Drive_Status == COMPLETE) Drive_Status = JOINT;
  if (Arm_Status   == COMPLETE) Arm_Status   = JOINT;

  if ((Drive_Status == JOINT) & (Arm_Status == JOINT))
  {
    if (startFlag)
    {
      startFlag = FALSE;
      duration = Clockcs + (unsigned long) time; // Current time + duration = cmd completed
//     PrintString((char *) "Delay duration = ");
//     PrintDecimal(duration);
    }

    if (duration > Clockcs)
    { // Keep waiting
      Joint_Status = INCOMPLETE;
    }
    else
    { // All done (move on to next drive command)
      Joint_Status = COMPLETE;
      Drive_Status = COMPLETE;
      Arm_Status   = COMPLETE;
      lastJointCmd = ENDCMD;  // In case there are two of the same type cmd in a row
    }
  }
}


/*******************************************************************************
* FUNCTION NAME: Synch_Drive_Arm
* PURPOSE:       Synchronize the Arm and Drive commands
*                Wait for both to COMPLETE, before proceeding
*******************************************************************************/
void Sync_Drive_Arm(int time)
{
  static unsigned long duration;
  static char startFlag=TRUE;

  if (lastJointCmd != SYNC)
  {
    lastJointCmd = SYNC;
    Joint_Status = INCOMPLETE;
    PrintString((char *) " **Synch_Drive_Arm** ");
    PrintCR();
  }

  /* Don't start until both drive and arm command queues are free */
  if (Drive_Status == COMPLETE) Drive_Status = JOINT;
  if (Arm_Status   == COMPLETE) Arm_Status   = JOINT;

  if ((Drive_Status == JOINT) & (Arm_Status == JOINT))
  {
    if (startFlag)
    {
      startFlag = FALSE;
      duration = Clockcs + (unsigned long) time; // Delay to allow previous operations to settle
    }

    if (duration > Clockcs)
    { // Delay to allow previous operations to settle
      Joint_Status = INCOMPLETE;
    }
    else
    {
      Joint_Status = COMPLETE;
      Drive_Status = COMPLETE;
      Arm_Status   = COMPLETE;
    }
  }
}


/*******************************************************************************
* FUNCTION NAME: Balance
* PURPOSE:       Diagnostic to measure the values needed to balance the two sides
*                of the drivetrain.
*******************************************************************************/
#define SETTLE 2
void Balance(int speed)
{
  static int RAdjust, LAdjust;
  static char firstRun, fastside, loop;
  long Rencoder, Lencoder;

  Rencoder = Right_Encoder_Count;
  Lencoder = Left_Encoder_Count;
  if (Rencoder < 0) Rencoder = -Rencoder;
  if (Lencoder < 0) Lencoder = -Lencoder;

//speed = 127;  // Cannot pass in for some buggy reason

//PrintString((char *) " speed=");
//PrintSignedDecimal(speed);

  if (lastDriveCmd != BALANCE)
  {
    lastDriveCmd = BALANCE;
    Drive_Status = INCOMPLETE;
    RAdjust = LAdjust = 0;
    firstRun = TRUE;
    loop = 0;
    PrintString((char *) " **BALANCE** ");
    PrintCR();

    /* Disable interrupts while we zero the shaft encoders */
    INTCONbits.GIEL = 0;
    Right_Encoder_Count = 0;
    Left_Encoder_Count = 0;
    INTCONbits.GIEL = 1;    
  }
  else
  {
    if (loop < SETTLE)
      loop++;
    else
    {
      loop = 0;
      if (firstRun)
      {
        firstRun = FALSE;
        // Is the right or left side faster?
        if (Rencoder > Lencoder)
          fastside = 1;
        else
          fastside = 0;        
      }

      if (fastside == 1)  // Adjust right side
      {
        if (Rencoder > Lencoder)
          RAdjust += 1;
        else if (Rencoder < Lencoder)
        {
          RAdjust -= 1;
          PrintString((char *) " *** RAdjust is close to ");
          PrintSignedDecimal(RAdjust);
        }
        else
        {
//          lastDriveCmd = ENDCMD;
//          Drive_Status = COMPLETE;
          PrintString((char *) " **** RAdjust is EXACTLY ");
          PrintSignedDecimal(RAdjust);
        }
        if (RAdjust > 127)
        {
          lastDriveCmd = ENDCMD;
          Drive_Status = COMPLETE;
          PrintString((char *) " **** RAdjust has NO solution **** ");
        }
      }
      else  // Adjust left side
      {
        if (Lencoder > Rencoder)
          LAdjust += 1;
        else if (Lencoder < Rencoder)
        {
          LAdjust -= 1;
          PrintString((char *) " LAdjust is close to ");
          PrintSignedDecimal(LAdjust);
        }
        else
        {
//          lastDriveCmd = ENDCMD;
//          Drive_Status = COMPLETE;
          PrintString((char *) " LAdjust is EXACTLY ");
          PrintSignedDecimal(LAdjust);
        }
        if (LAdjust > 127)
        {
          lastDriveCmd = ENDCMD;
          Drive_Status = COMPLETE;
          PrintString((char *) " **** LAdjust has NO solution **** ");
        }
      } // fastside
    }  // loop
    /* Reset Encoders */
    /* Disable interrupts while we zero the shaft encoders */
    INTCONbits.GIEL = 0;
    Right_Encoder_Count = 0;
    Left_Encoder_Count = 0;
    INTCONbits.GIEL = 1;    

    if (fastside == 1) // Adjust right side
    {
      WHL_L1 = speed;
      if (speed < 0)
        WHL_R1 = speed + RAdjust;
      else
        WHL_R1 = speed - RAdjust;
      if ((WHL_R1 > 127) | (WHL_R1 <-127))
        {
          lastDriveCmd = ENDCMD;
          Drive_Status = COMPLETE;
          PrintString((char *) " **** WHL_R1 has NO solution **** ");
        }
    }
    else  // Adjust left side
    {
      WHL_R1 = speed;
      if (speed < 0)
        WHL_L1 = speed + LAdjust;
      else
        WHL_L1 = speed - LAdjust;
      if ((WHL_L1 > 127) | (WHL_L1 <-127))
        {
          lastDriveCmd = ENDCMD;
          Drive_Status = COMPLETE;
          PrintString((char *) " **** WHL_L1 has NO solution **** ");
        }
    }

    PrintString((char *) " L1=");
    PrintSignedDecimal(WHL_L1);
    PrintString((char *) " R1=");
    PrintSignedDecimal(WHL_R1);
    PrintCR();
  } // initialize
}
