/*******************************************************************************
* FILE NAME: auto_scripts.c
*
*	Version 5.0	last updated 3/17/04::
*
* DESCRIPTION:
*  These are all the autonomous command scripts
*  The auto script is selected via the 16 position auto mode switch on the robot and
*  the left/right of the field switch.
*  The left/right scripts are not simply reversed because the mobile goal and it's
*  utility to our alliance changes with the side of the field.
*
* USAGE:
*  Modify scripts as required
*  Cut & Paste command templates located at the bottom of this file
*
*  Drive commands must be intermixed with Arm commands carefully.
*  Two of either type in a row will block further command processing
*  until they have cleared the queue.
*
*******************************************************************************/

//#include "auto_routines.h"

/****************************************************************************
 *                                                                          *
 *     This is where we load the different versions of the autonomous       *
 *                                                                          *
 ****************************************************************************/

#define AUTO_HEIGHT		20  // Maximum number of script commands
#define AUTO_WIDTH		4   // Maximum number of command parameters

/*-----------------------------------------------------------------------------*/
/*-------------------------ASSUME LEFT SIDE OF FIELD---------------------------*/
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_0[AUTO_HEIGHT*AUTO_WIDTH] = 
  { /* KEEP - Do nothing */
  ENDCMD, NONE,  NONE,       NONE     //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_1[AUTO_HEIGHT*AUTO_WIDTH] = 
  {/* position at goal */
   DFWD,       3,   QTRSPEED,      0,  // Drive Forward a distance (~18 on 2x ball)
   ARMOPN,     0,        0,        0,  // Open the 2x ball arms
   ARMFWD,    67,   FULLSPEED,     0,  // Arms forward for a time (69 on 2x ball)
   DFWD,       3,   HALFSPEED,     0,  // Drive Forward a distance (~18 on 2x ball)
   DFWD,      20,   FULLSPEED,     0,  // Drive Forward a distance 
  ENDCMD, NONE,  NONE,       NONE     //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_2[AUTO_HEIGHT*AUTO_WIDTH] = 
  { /* 10 pt ball take-down */
   DFWD,      28,   FULLSPEED,     0,  // Drive Forward a distance (was 26)
   ARMOPN,     0,        0,        0,  // Open the 2x ball arms
   ARMFWD,    12,   FULLSPEED,     0,  // Arms forward for a time (12 on 10pt ball)
   DSPIN,      8,   FULLSPEED,    CW,   // Spin in place (7 just shy)
   SYNC,       0,        0,        0,  // Wait for both Drive and Arm cmds to complete before proceeding
   DBCK,       6,   HALFSPEED,     0,  // Drive Backward a distance 
   DFWD,       6,   HALFSPEED,     0,  // Drive Forward a distance 
   DSPIN,      8,   FULLSPEED,    CCW,   // Spin in place (7 just shy)
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_3[AUTO_HEIGHT*AUTO_WIDTH] = 
  { /*  Grab goal & block 10 pt ball */
   DFWD,       3,   QTRSPEED,      0,  // Drive Forward a distance (~18 on 2x ball)
   ARMOPN,     0,        0,        0,  // Open the 2x ball arms
   ARMFWD,    63,   FULLSPEED,     0,  // Arms forward for a time (69 on 2x ball)
   DFWD,       3,   HALFSPEED,     0,  // Drive Forward a distance (~18 on 2x ball)
   DFWD,      25,   FULLSPEED,     0,  // Drive Forward a distance 
   GOALGRAB,   0,        0,        0,  // Grab a goal
   DFWD,      10,   HALFSPEED,     0,  // Drive Forward a distance 
   DFWD,      14,   FULLSPEED,     0,  // Drive Forward a distance 
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_4[AUTO_HEIGHT*AUTO_WIDTH] = 
  {/* goal grab w/slow start & pull back */
   DFWD,       3,   QTRSPEED,      0,  // Drive Forward a distance (~18 on 2x ball)
   ARMOPN,     0,        0,        0,  // Open the 2x ball arms
   ARMFWD,    63,   FULLSPEED,     0,  // Arms forward for a time (69 on 2x ball)
   DFWD,       3,   HALFSPEED,     0,  // Drive Forward a distance (~18 on 2x ball)
   DFWD,      24,   FULLSPEED,     0,  // Drive Forward a distance 
   GOALGRAB,   0,        0,        0,  // Grab a goal
   DFWD,      10,   HALFSPEED,     0,  // Drive Forward a distance 
   DBCK,       8,   FULLSPEED,     0,  // Drive Backward a distance 
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_5[AUTO_HEIGHT*AUTO_WIDTH] = 
  { /* position at mobile goal & grab 2x ball */
   DFWD,      25,   FULLSPEED,     0,  // Drive Forward a distance (~18 on 2x ball)
   ARMOPN,     0,         0,       0,  // Open the 2x ball arms
   ARMFWD,    65,   FULLSPEED,     0,  // Arms forward for a time (69 on 2x ball)
   SYNC,     0,         0,         0,    // Wait for both Drive and Arm cmds to complete before proceeding
   ARMCLS,     0,         0,        0,  // Close the 2x ball arms
   ARMBCK,   5,    FULLSPEED,    0,  // Arms backward for a time
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_6[AUTO_HEIGHT*AUTO_WIDTH] = 
  { /* 10 pt. ball -- back into the ball */
   DFWD,      26,   FULLSPEED,     0,  // Drive Forward a distance (~18 on 2x ball)
   ARMOPN,     0,       0,         0,  // Open the 2x ball arms
   ARMFWD,   20 ,   FULLSPEED,     0,  // Arms forward for a time (78 on 10pt ball)
   GTURN,    90,        0,         0,  // Turn to a heading (gyro)
   SYNC,     0,         0,         0,    // Wait for both Drive and Arm cmds to complete before proceeding
   DBCK,     6,     FULLSPEED,     0,  // Drive Backward a distance (~18 on 2x ball)
   GTURN,    100,       0,         0,  // Turn to a heading (gyro)
   GTURN,    80,        0,         0,  // Turn to a heading (gyro)
   GTURN,    90,        0,         0,  // Turn to a heading (gyro)
  ENDCMD, NONE,  NONE,       NONE    //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_7[AUTO_HEIGHT*AUTO_WIDTH] = 
  {
   DFWD,     27,    FULLSPEED,     0,  // Drive Forward a distance (~18 on 2x ball)
   ARMOPN,    0,         0,        0,  // Open the 2x ball arms
   ARMFWD,   65,    FULLSPEED,     0,  // Arms forward for a time (69 on 2x ball)
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_8[AUTO_HEIGHT*AUTO_WIDTH] = 
  {
   DFWD,     27,    FULLSPEED,     0,  // Drive Forward a distance (~18 on 2x ball)
   ARMOPN,    0,         0,        0,  // Open the 2x ball arms
   ARMFWD,   65,    FULLSPEED,     0,  // Arms forward for a time (69 on 2x ball)
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_9[AUTO_HEIGHT*AUTO_WIDTH] = 
  {
   DFWD,     27,    FULLSPEED,     0,  // Drive Forward a distance (~18 on 2x ball)
   ARMOPN,    0,         0,        0,  // Open the 2x ball arms
   ARMFWD,   65,    FULLSPEED,     0,  // Arms forward for a time (69 on 2x ball)
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_10[AUTO_HEIGHT*AUTO_WIDTH] = 
  {
   DBCK,     10,   FULLSPEED,     0,   // Drive Backward for a time
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_11[AUTO_HEIGHT*AUTO_WIDTH] = 
  {
   DFWD,     20,    FULLSPEED,     0,  // Drive Forward a distance (~18 on 2x ball)
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_12[AUTO_HEIGHT*AUTO_WIDTH] = 
  {
   DSPIN,    8,   FULLSPEED,    CW,   // Spin in place (7 just shy)
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_13[AUTO_HEIGHT*AUTO_WIDTH] = 
  { /* Test all commands */
   GOALGRAB,    0,        0,        0,    // Grab a goal
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_14[AUTO_HEIGHT*AUTO_WIDTH] = 
  {
   GTURN,    90,   0,        0,  // Turn to a heading (gyro)
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/
rom const	int  AUTO_15[AUTO_HEIGHT*AUTO_WIDTH] = 
  { // Do nothing
   BALANCE,  0,     HALFSPEED,       0,  // Measure the left/right balance for the drive train
  ENDCMD, NONE,  NONE,       NONE  //MUST be the last command
  };
/*-----------------------------------------------------------------------------*/

  /******** End autonomous commands ********/
  /*****************************************/
  /*****************************************/

/*******************************************************************************
* Available Commands:
*   !!!!!!!!! This must ALWAYS be the last command in a script !!!!!!!!!!
   ENDCMD,      0,           0,     0,   // End of the Command script
*
* Driving :
*(timed) - 87"/sec startup, 112.5"/sec from then on
   DFWD,     TIME,   FULLSPEED,     0,   // Drive Forward for a time
   DSPIN,    TIME,   FULLSPEED,    CW,   // Spin in place for a time CW or CCW
   DBCK,     TIME,   FULLSPEED,     0,   // Drive Backward for a time
* (sensor based)
   GTURN,  GYROHEADING,   0,        0,  // Turn to a heading (gyro)
   GOALGRAB,    TIME,        0,        0,    // Grab a goal
   GOALREL,     TIME,        0,        0,    // Release a goal
*
* ARMS:
   ARMOPN,     0,         0,        0,  // Open the 2x ball arms
   ARMCLS,     0,         0,        0,  // Close the 2x ball arms
*(timed)
   ARMFWD,   TIME,    FULLSPEED,    0,  // Arms forward 1.2sec=28"(10pt ball),2.2sec=58"(2X ball on mobile goal)
   ARMBCK,   TIME,    FULLSPEED,    0,  // Arms backward for a time
*
* JOINT:
   DELAY,    TIME,        0,        0,    // Sit still for a time
   SYNC,     TIME,        0,        0,    // Wait for both Drive and Arm cmds to complete before proceeding
*
* FUTURE COMMANDS:
   IRTURN, IRHEADING,     0,        0,  // Turn to a servo heading relative to IR beacon
   CURVE, DISTRIGHT, FULLSPEED, DISTLEFT,  // Follow a curved path
   DISTTURN, DIST,   FULLSPEED,    CW,  // Turn a distance CW or CCW
*
* BROKEN COMMANDS:
*(sensor based)
   ARMSET,  SETTING,  FULLSPEED,    0,  // Arms to a specified position
*(distance)
   DISTFWD,  DIST,   FULLSPEED,     0,  // Drive Forward a distance
   DISTBCK,  DIST,   FULLSPEED,     0,  // Drive Backward a distance
*
* DIAGNOSTIC COMMANDS:
   BALANCE,  0,      FULLSPEED,     0,  // Measure the left/right balance for the drive train
*
* PARAMETERS:
*  TIME: 1-150 tenths of seconds
*  Distance: 1-576 inches
*  Direction: CW or CCW
*  Speed: 1-127
*  Arm Setting: 0-50
*  Gyro heading: -179 to 180, 0 is the original robot heading straight backward
*  IR heading: -127 to 127 servo position 0 is straight backward
*  Arm timed moves- 1sec=25",2sec=53",3sec=85",4sec=109",7sec=2Xball,8sec=19"
*                   1.1sec=25",1.2sec=28",1.3sec=33",2.2sec=58"
*                   1.2sec=28"(10pt ball-bkwds)
*                   2.2sec=58"(2Xball-bkwds)
*                   7.0sec=2X ball frwds
*  DSPIN times:  2sec=260 degrees,  .8sec = 90 degrees
*******************************************************************************/
