/*******************************************************************************
* FILE NAME: user_routines.c
*
* DESCRIPTION:
*  This file contains the default mappings of inputs  
*  (like switches, joysticks, and buttons) to outputs on the RC.  
*
* USAGE:
*  You can either modify this file to fit your needs, or remove it from your 
*  project and replace it with a modified copy. 
*  
* Notes: digital input pins are "high" or 1 when open and "low" or 0 when closed
*        OI inputs are TRUE or "1" when the button is pressed
*
* SPECIAL NOTE:
*  Compiler flags used in this project are:
*  _FRC_BOARD compiles the FRC version of certain routine calls
*  _NO_PRINTS disables all serial output print statements used
*  _SIMULATOR disables non-MPLAB SIM compliant code
*
*******************************************************************************/

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
//#include "receiver.h"

#include "timers.h"

#include "clock_routines.h"
#include "sensor_routines.h"

#define VARIABLE_DECLARE  /* causes certain global variables to be declared only once */
//  #include "tracker.h"
  #include "user_routines.h"
  #include "auto_routines.h"
  #include "motor_routines.h"
  #include "print_routines.h"
#undef VARIABLE_DECLARE

//void Test_Drive (void);

/*******************************************************************************
* FUNCTION NAME: User_Initialization
* PURPOSE:       This routine is called first (and only once) in the Main function.  
*                You may modify and add to this function.
* CALLED FROM:   main.c
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void User_Initialization (void)
{

  Set_Number_of_Analog_Channels(SIXTEEN_ANALOG);    /* DO NOT CHANGE! */

/* FIRST: Set up the I/O pins you want to use as digital INPUTS. */
  digital_io_01 = digital_io_02 = digital_io_03 = digital_io_04 = INPUT;
  digital_io_05 = digital_io_06 = digital_io_07 = digital_io_08 = INPUT;
  digital_io_09 = digital_io_10 = digital_io_11 = digital_io_12 = INPUT;
  digital_io_13 = digital_io_14 = digital_io_15 = digital_io_16 = INPUT;
  digital_io_17 = INPUT;  digital_io_18 = INPUT;  /* Used for pneumatic pressure switch. */

/* SECOND: Set up the I/O pins you want to use as digital OUTPUTS. */

/* THIRD: Initialize the values on the digital outputs. */

/* FOURTH: Set your initial PWM values.  Neutral is 127. */
  pwm01 = pwm02 = pwm03 = pwm04 = pwm05 = pwm06 = pwm07 = pwm08 = 127;
  pwm09 = pwm10 = pwm11 = pwm12 = pwm13 = pwm14 = pwm15 = pwm16 = 127;

/* FIFTH: Set your PWM output types for PWM OUTPUTS 13-16.
  /*   Choose from these parameters for PWM 13-16 respectively:               */
  /*     IFI_PWM  - Standard IFI PWM output generated with Generate_Pwms(...) */
  /*     USER_CCP - User can use PWM pin as digital I/O or CCP pin.           */

  Setup_PWM_Output_Type(IFI_PWM,IFI_PWM,IFI_PWM,IFI_PWM);

  /* 
     Example: The following would generate a 40KHz PWM with a 50% duty cycle on the CCP2 pin:

         CCP2CON = 0x3C;
         PR2 = 0xF9;
         CCPR2L = 0x7F;
         T2CON = 0;
         T2CONbits.TMR2ON = 1;

         Setup_PWM_Output_Type(USER_CCP,IFI_PWM,IFI_PWM,IFI_PWM);
  */

  /* Add any other initialization code here. */
  /* Initialize relay outputs */
    relay1_fwd = 0; relay1_rev = 0;
    relay2_fwd = 0; relay2_rev = 0; // 2X  ball grabber
    relay3_fwd = 0; relay3_rev = 0; // Wings
    relay4_fwd = 0; relay4_rev = 0; // Goal grabber
    relay5_fwd = 0; relay5_rev = 0;
    relay6_fwd = 0; relay6_rev = 0;
    relay7_fwd = 0; relay7_rev = 0;
    relay8_fwd = 0; relay8_rev = 0; // Compressor

  Initialize_Clock();

  Initialize_Serial_Comms();   //Use no "print" statement before the serial comms initialize
  
  Initialize_Interrupts();

//  Initialize_Receiver();

//  Initialize_Tracker();

  Putdata(&txdata);             /* DO NOT CHANGE! */

  User_Proc_Is_Ready();         /* DO NOT CHANGE! - last line of User_Initialization */
}

/*******************************************************************************
* FUNCTION NAME: Process_Data_From_Master_uP
* PURPOSE:       Executes every 26.2ms when it gets new data from the master 
*                microprocessor.
* CALLED FROM:   main.c
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void Process_Data_From_Master_uP(void)
{
  Current_Sensor(); // Update the drive motor amps
  pwm01 = pwm02 = pwm03 = pwm04 = pwm05 = pwm06 = pwm07 = pwm08 = 127;
  pwm09 = pwm10 = pwm11 = pwm12 = pwm13 = pwm14 = pwm15 = pwm16 = 127;

}

/*******************************************************************************
* FUNCTION NAME: User_Driver
* PURPOSE:       Performs the default mappings of inputs to outputs for the
*                Robot Controller.
* CALLED FROM:   this file, Process_Data_From_Master_uP routine
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void User_Driver(void)
{
  static char BallToggle=0, WingToggle=0, GoalToggle=0;
  static char counter = 0;

counter++;
if (counter > 40)
{
counter = 0;
PrintString((char *) " ArmSetting=");
PrintDecimal(ArmSetting);
}

  if (p2_sw_aux1)
  {
    /*---------- 1 Joystick Drive ----------------------------------------------
     *--------------------------------------------------------------------------
     *  This code mixes the Y and X axis on Port 1 to allow one joystick drive. 
     *  Joystick forward  = Robot forward
     *  Joystick backward = Robot backward
     *  Joystick right    = Robot rotates right
     *  Joystick left     = Robot rotates left
     *  Connect the right drive motors to PWM13 and/or PWM14 on the RC.
     *  Connect the left  drive motors to PWM15 and/or PWM16 on the RC.
     */  

   //  Smooth_Joystick(&ip1_y);
   //  Smooth_Joystick(&ip1_x);

     Continuous_Deadband(&ip4_y);

   //  ipwm16 = ipwm15 = -Limit_Mix(ip1_y + ip1_x);
   //  ipwm14 = ipwm13 = -Limit_Mix(ip1_y - ip1_x);

     WHL_R1 = WHL_R2 = -Limit_Mix(ip4_y + ip4_x);
     WHL_L1 = WHL_L2 = -Limit_Mix(ip4_y - ip4_x);
  }
  else
  {
   /*---------- 2 Joystick Drive ----------------------------------------------
    *--------------------------------------------------------------------------
    */
  //  Smooth_Joystick(&ip1_y);
  //  Smooth_Joystick(&ip2_y);

    Continuous_Deadband(&ip3_y);
    Continuous_Deadband(&ip4_y);

    WHL_R1 = WHL_R2 = -ip4_y;
    WHL_L1 = WHL_L2 = -ip3_y;
  }

 /*---------- Hold Stationary Position --------------------------------------
  *--------------------------------------------------------------------------
  */

 /*---------- Arm -----------------------------------------------------------
  *--------------------------------------------------------------------------
  */
  Continuous_Deadband(&ip1_y);
  ARM_PWM = ip1_y;


 /*-------------- Winch -----------------------------------------------------
  *--------------------------------------------------------------------------
  */

    /* use p2_wheel to set the hook? */
    if (WINCH_UP_INPUT & WINCH_DOWN_INPUT)  // Lower da bot
    {
      WINCH_PWM1 = WINCH_PWM2 = 127;
    }
    else if (WINCH_UP_INPUT | WINCH_DOWN_INPUT)  // hoist da bot
    {
      WINCH_PWM1 = WINCH_PWM2 = -127;
    }
    else
    {
      WINCH_PWM1 = WINCH_PWM2 = 0; // stop all engines
    }

 /*------------- Solenoids --------------------------------------------------
  *--------------------------------------------------------------------------
  */
  /** 2X Ball grabber **/
  if (!BALL2X_GRABBER_INPUT)
  {
    BallToggle = 1;
  }
  if (BALL2X_GRABBER_INPUT & BallToggle)
  {
    BALL2X_GRABBER = !BALL2X_GRABBER;
    relay2_rev = 0;
    BallToggle = 0;
  }

  /** Wings **/
//  if (WINGS_INPUT)
//  {
//    WingToggle = 1;
//  }
//  if (!WINGS_INPUT & WingToggle)
//  {
//    WINGS = !WINGS;
//    relay4_rev = 0;
//    WingToggle = 0;
//  }

   /** Toggle goal grabber **/
  /** Goal grabber **/
//  if (GOAL_GRABBER_INPUT)  // Change on button release
  if (!GOAL_GRABBER_INPUT)  // Change on button press
  {
    GoalToggle = 1;
  }
//  if (!GOAL_GRABBER_INPUT & GoalToggle)  // Change on button release
  if (GOAL_GRABBER_INPUT & GoalToggle) // Change on button press
  {
    GOAL_GRABBER = !GOAL_GRABBER;
    relay3_rev = 0;
    GoalToggle = 0;
  }

  /** goal grabber down only while trigger is held **/
    /* OI switch is true when the switch is depressed */
//    if (GOAL_GRABBER_INPUT) // Leave goal in current position until first user action
//      AutoGoal = TRUE;

//    if (AutoGoal) // Leave goal in current position until first user action
//    {
//      GOAL_GRABBER = GOAL_GRABBER_INPUT;
//      relay3_rev = 0;
//    }
   

 /*---------- Buttons to Relays----------------------------------------------
  *--------------------------------------------------------------------------
  */
  /* Might not want to allow for the first n sec to avoid interferring with the gyro init */
  /* Not an issue if robot is started in disabled mode */
  COMPRESSOR = !rc_dig_in14;  /* Power pump only if pressure switch is off. */
  relay8_rev = 0;
  
 /*---------- Update the OI LEDs and dashboard display ----------------------
  *--------------------------------------------------------------------------
  */
  Update_OI();

} /* END Default_Routine(); */

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
