/*******************************************************************************
* FILE NAME: user_routines.h
*
* DESCRIPTION: 
*  This is the include file which corresponds to user_routines.c and
*  user_routines_fast.c
*  It contains some aliases and function prototypes used in those files.
*
* USAGE:
*  If you add your own routines to those files, this is a good place to add
*  your custom macros (aliases), type definitions, and function prototypes.
*******************************************************************************/

#ifndef __user_routines_h_
#define __user_routines_h_

#ifdef VARIABLE_DECLARE
      /** One encoder revolution (256 interrupts) = 11.25" of travel **/
      /*  One encoder "tick" = .044" */
  signed long Right_Encoder_Count=0,Left_Encoder_Count=0;
  char Beacon_type=0;
  int ArmSetting=0; // Where is the arm now?
  char AutoGoal=0; // FALSE by default, goal waits for first user trigger
#else
  extern signed long Right_Encoder_Count,Left_Encoder_Count; //
  extern char Beacon_type;
  extern int ArmSetting;
  extern char AutoGoal;
#endif

/*******************************************************************************
                            MACRO DECLARATIONS
*******************************************************************************/

/* Used in limit switch routines in user_routines.c */
#define OPEN        1     /* Limit switch is open (input is floating high). */
#define CLOSED      0     /* Limit switch is closed (input connected to ground). */

#define FALSE             0
#define TRUE              1
#define ON                1
#define OFF               0

#define WHL_L1	ipwm09
#define WHL_L2	ipwm10
#define WHL_R1	ipwm11
#define WHL_R2	ipwm12

#define ARM_PWM	ipwm03

#define WINCH_PWM1	ipwm04
#define WINCH_PWM2	ipwm05

/* On DRIVE joystick */
#define GOAL_GRABBER    relay4_fwd
#define WINGS           relay3_fwd

#define GOAL_GRABBER_INPUT    p4_sw_trig
#define WINGS_INPUT           p4_sw_top

/* On ARM joystick */
#define BALL2X_GRABBER  relay2_fwd
#define COMPRESSOR      relay8_fwd

#define BALL2X_GRABBER_INPUT  p1_sw_trig
#define WINCH_UP_INPUT	      p1_sw_aux2
#define WINCH_DOWN_INPUT	    p1_sw_aux1


/*******************************************************************************
                            TYPEDEF DECLARATIONS
*******************************************************************************/


/*******************************************************************************
                           FUNCTION PROTOTYPES
*******************************************************************************/

/* These routines reside in user_routines.c */
void User_Initialization(void);
void Process_Data_From_Master_uP(void);
void User_Driver(void);
void Update_OI(void);

/* These routines reside in user_routines_fast.c */
void InterruptHandlerLow (void);  /* DO NOT CHANGE! */
void User_Autonomous_Code(void);  /* Only in full-size FRC system. */
void Process_Data_From_Local_IO(void);
void IR_Sensor_Tracking(void);
void Where_Is_The_Arm(void);

#endif
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
